@@ -37,6 +37,8 @@ NULL
 ##' @aliases EMci_biomod2_model-class
 ##' @aliases EMca_biomod2_model-class
 ##' @aliases EMwmean_biomod2_model-class
+##' @aliases EMmode_biomod2_model-class
+##' @aliases EMfreq_biomod2_model-class
 ##' @author Damien Georges
 ##' 
 ##' @title Ensemble model output object class (when running \code{BIOMOD_EnsembleModeling()})
@@ -74,6 +76,8 @@ NULL
 ##'   \item \code{EMci_biomod2_model} : \code{model_class} is \code{EMci}
 ##'   \item \code{EMca_biomod2_model} : \code{model_class} is \code{EMca}
 ##'   \item \code{EMwmean_biomod2_model} : \code{model_class} is \code{EMwmean}
+##'   \item \code{EMmode_biomod2_model} : \code{model_class} is \code{EMmode}
+##'   \item \code{EMfreq_biomod2_model} : \code{model_class} is \code{EMfreq}
 ##' }
 ##' 
 ##' 
@@ -90,6 +94,8 @@ NULL
 ##' showClass("EMci_biomod2_model")
 ##' showClass("EMca_biomod2_model")
 ##' showClass("EMwmean_biomod2_model")
+##' showClass("EMmode_biomod2_model")
+##' showClass("EMfreq_biomod2_model")
 ##' 
 ##' 
 NULL
@@ -114,10 +120,7 @@ setClass('biomod2_ensemble_model',
 ##' @export
 ##' 
 
-setMethod('show', signature('biomod2_ensemble_model'),
-          function(object) {
-            callNextMethod(object)
-          })
+setMethod('show', signature('biomod2_ensemble_model'), function(object) { callNextMethod(object) })
 
 
 ## 9.3 predict2 method -------------------------------------------------------------
@@ -139,7 +142,11 @@ setMethod('show', signature('biomod2_ensemble_model'),
 ##' @aliases predict2.EMca_biomod2_model.data.frame
 ##' @aliases predict2.EMwmean_biomod2_model.SpatRaster
 ##' @aliases predict2.EMwmean_biomod2_model.data.frame
-##' @author Remi Patin
+##' @aliases predict2.EMmode_biomod2_model.SpatRaster
+##' @aliases predict2.EMmode_biomod2_model.data.frame
+##' @aliases predict2.EMfreq_biomod2_model.SpatRaster
+##' @aliases predict2.EMfreq_biomod2_model.data.frame
+##' @author Rémi Lemaire-Patin
 ##' 
 ##' @title Functions to get predictions from \code{\link{biomod2_ensemble_model}} objects
 ##' 
@@ -167,10 +174,13 @@ setMethod('show', signature('biomod2_ensemble_model'),
 
 NULL
 
-### biomod2_ensemble_model + SpatRaster  -------------------------------------------------
+##' 
 ##' @rdname predict2.em
+##' 
+
 setMethod('predict2', signature(object = 'biomod2_ensemble_model', newdata = "SpatRaster"),
-          function(object, newdata, predfun, seedval = NULL, ...) {
+          function(object, newdata, predfun, seedval = NULL, ...)
+          {
             args <- list(...)
             data_as_formal_predictions <- args$data_as_formal_predictions
             if (is.null(data_as_formal_predictions)) {
@@ -181,21 +191,15 @@ setMethod('predict2', signature(object = 'biomod2_ensemble_model', newdata = "Sp
             on_0_1000 <- args$on_0_1000
             na.rm <- args$na.rm
             
-            if (is.null(na.rm)) { 
-              na.rm <- TRUE 
-            }
-            
-            if (is.null(overwrite)) { 
-              overwrite <- TRUE 
-            }
-            if (is.null(on_0_1000)) { 
-              on_0_1000 <- FALSE
-            }
+            if (is.null(na.rm)) { na.rm <- TRUE }
+            if (is.null(overwrite)) { overwrite <- TRUE }
+            if (is.null(on_0_1000)) { on_0_1000 <- FALSE }
             
             # additional arg retrieved for EMci
             sd_prediction <- args$sd_prediction
             mean_prediction <- args$mean_prediction
             side <- args$side
+            data.type <- args$data.type
             # additional arg retrived for EMca
             thresh <- args$thresh
             # additional arg retrived for EMwmean
@@ -207,6 +211,7 @@ setMethod('predict2', signature(object = 'biomod2_ensemble_model', newdata = "Sp
               newdata <- subset(newdata, object@model)
             } else {
               newdata <- .get_formal_predictions(object, newdata, on_0_1000 = on_0_1000, seedval = seedval)
+              #newdata <- apply(newdata, as.numeric) ## ? normalement pas utile ?
             }
             
             out <- predfun(newdata, 
@@ -214,29 +219,33 @@ setMethod('predict2', signature(object = 'biomod2_ensemble_model', newdata = "Sp
                            mean_prediction = mean_prediction,
                            sd_prediction = sd_prediction, 
                            side = side,
+                           data.type = data.type,
                            thresh = thresh, 
                            penalization_scores = penalization_scores,
                            mod.name = mod.name,
                            na.rm = na.rm)
             
+            
             if (!is.null(out) & !is.null(filename)) {
               cat("\n\t\tWriting projection on hard drive...")
               if (on_0_1000 & !inherits(object, "EMcv_biomod2_model")) { ## projections are stored as positive integer
-                writeRaster(out, filename = filename, overwrite = overwrite, 
-                            datatype = "INT2S", NAflag = -9999)
+                writeRaster(out, filename = filename, overwrite = overwrite, datatype = "INT2S", NAflag = -9999)
               } else { ## keep default data format for saved raster
                 writeRaster(out, filename = filename, overwrite = overwrite)
               }
               out <- rast(filename)
             }
             return(out)
-            
-          })
+          }
+)
 
-### biomod2_ensemble_model + data.frame  -------------------------------------
+##' 
 ##' @rdname predict2.em
+##' 
+
 setMethod('predict2', signature(object = 'biomod2_ensemble_model', newdata = "data.frame"),
-          function(object, newdata, predfun, seedval = NULL,  ...) {
+          function(object, newdata, predfun, seedval = NULL,  ...)
+          {
             args <- list(...)
             data_as_formal_predictions <- args$data_as_formal_predictions
             if (is.null(data_as_formal_predictions)) {
@@ -244,19 +253,15 @@ setMethod('predict2', signature(object = 'biomod2_ensemble_model', newdata = "da
             }
             
             on_0_1000 <- args$on_0_1000
-            if (is.null(on_0_1000)) {
-              on_0_1000 <- FALSE 
-            }
-            
+            if (is.null(on_0_1000)) { on_0_1000 <- FALSE }
             na.rm <- args$na.rm
-            if (is.null(na.rm)) { 
-              na.rm <- TRUE 
-            }
+            if (is.null(na.rm)) { na.rm <- TRUE }
             
             # additional arg retrieved for EMci
             sd_prediction <- args$sd_prediction
             mean_prediction <- args$mean_prediction
             side <- args$side
+            data.type <- args$data.type
             # additional arg retrived for EMca
             thresh <- args$thresh
             # additional arg retrived for EMwmean
@@ -264,20 +269,27 @@ setMethod('predict2', signature(object = 'biomod2_ensemble_model', newdata = "da
             
             if (data_as_formal_predictions) { 
               newdata <- newdata[ , object@model, drop = FALSE]
-            } else  {
+            } else {
               newdata <- .get_formal_predictions(object, newdata, on_0_1000 = on_0_1000, seedval = seedval)
+              if (object@model_class %in% c("EMmode", "EMfreq")){
+                newdata <- apply(newdata, 2, as.character)
+              } else {
+                newdata <- apply(newdata, 2, as.numeric)
+              }
             }
+            
             out <- predfun(newdata,
                            on_0_1000 = on_0_1000,
                            mean_prediction = mean_prediction,
                            sd_prediction = sd_prediction,
                            side = side,
+                           data.type = data.type,
                            thresh = thresh, 
                            penalization_scores = penalization_scores,
                            na.rm = na.rm)
             return(out)
-            
-          })
+          }
+)
 
 # --------------------------------------------------------------------------- #
 # 10.1 biomod2_ensemble_model subclass ---------------------------------------
@@ -290,7 +302,6 @@ setMethod('predict2', signature(object = 'biomod2_ensemble_model', newdata = "da
 ##' @rdname biomod2_ensemble_model
 ##' @export
 
-
 setClass('EMmean_biomod2_model',
          representation(),
          contains = 'biomod2_ensemble_model',
@@ -302,17 +313,16 @@ setClass('EMmean_biomod2_model',
 ##' 
 
 setMethod('predict2', signature(object = 'EMmean_biomod2_model', newdata = "SpatRaster"),
-          function(object, newdata, ...) {
-            predfun <- function(newdata, on_0_1000, mod.name, na.rm, ...){
+          function(object, newdata, ...)
+          {
+            predfun <- function(newdata, on_0_1000, mod.name, na.rm, ...) {
               if (nlyr(newdata) == 1) {
                 return(newdata)
               } else {
                 return(
-                  app(newdata,function(x){
+                  app(newdata, function(x) {
                     m <- mean(x, na.rm = na.rm)
-                    if (on_0_1000) { 
-                      m <- round(m)
-                    }
+                    if (on_0_1000) { m <- round(m) }
                     return(m)
                   }, wopt = list(names = mod.name))
                 )
@@ -323,14 +333,16 @@ setMethod('predict2', signature(object = 'EMmean_biomod2_model', newdata = "Spat
           }
 )
 
+##' 
 ##' @rdname predict2.em
+##' 
+
 setMethod('predict2', signature(object = 'EMmean_biomod2_model', newdata = "data.frame"),
-          function(object, newdata, ...) {
-            predfun <- function(newdata, on_0_1000, na.rm, ...){
+          function(object, newdata, ...)
+          {
+            predfun <- function(newdata, on_0_1000, na.rm, ...) {
               out <- rowMeans(newdata, na.rm = na.rm)
-              if (on_0_1000) { 
-                out <- round(out) 
-              }
+              if (on_0_1000) { out <- round(out) }
               out
             }
             # redirect to predict2.biomod2_ensemble_model.SpatRaster
@@ -356,19 +368,17 @@ setClass('EMmedian_biomod2_model',
 ##' @rdname predict2.em
 ##' 
 
-
 setMethod('predict2', signature(object = 'EMmedian_biomod2_model', newdata = "SpatRaster"),
-          function(object, newdata, ...) {
-            predfun <- function(newdata, on_0_1000, mod.name, na.rm, ...){
+          function(object, newdata, ...)
+          {
+            predfun <- function(newdata, on_0_1000, mod.name, na.rm, ...) {
               if (nlyr(newdata) == 1) {
                 return(newdata)
               } else {
                 return(
-                  app(newdata,function(x){
+                  app(newdata, function(x) {
                     m <- median(x, na.rm = na.rm)
-                    if (on_0_1000) { 
-                      m <- round(m)
-                    }
+                    if (on_0_1000) { m <- round(m) }
                     return(m)
                   }, wopt = list(names = mod.name))
                 )
@@ -379,14 +389,16 @@ setMethod('predict2', signature(object = 'EMmedian_biomod2_model', newdata = "Sp
           }
 )
 
+##' 
 ##' @rdname predict2.em
+##' 
+
 setMethod('predict2', signature(object = 'EMmedian_biomod2_model', newdata = "data.frame"),
-          function(object, newdata, ...) {
-            predfun <- function(newdata, on_0_1000, na.rm, ...){
+          function(object, newdata, ...)
+          {
+            predfun <- function(newdata, on_0_1000, na.rm, ...) {
               out <- apply(newdata, 1, median, na.rm = na.rm)
-              if (on_0_1000) { 
-                out <- round(out) 
-              }
+              if (on_0_1000) { out <- round(out) }
               out
             }
             # redirect to predict2.biomod2_ensemble_model.SpatRaster
@@ -412,13 +424,14 @@ setClass('EMcv_biomod2_model',
 ##' 
 
 setMethod('predict2', signature(object = 'EMcv_biomod2_model', newdata = "SpatRaster"),
-          function(object, newdata,  ...) {
-            predfun <- function(newdata, on_0_1000, mod.name, na.rm, ...){
+          function(object, newdata,  ...)
+          {
+            predfun <- function(newdata, on_0_1000, mod.name, na.rm, ...) {
               if (nlyr(newdata) <= 1) {
                 stop(paste0("\n Model EMcv was not computed because only one single model was kept in ensemble modeling (", names(newdata), ")"))
               }
-              out <- app(newdata, function(x){
-                sd(x, na.rm = na.rm)/mean(x, na.rm = na.rm) * 100
+              out <- app(newdata, function(x) {
+                sd(x, na.rm = na.rm) / mean(x, na.rm = na.rm) * 100
               }, wopt = list(names = mod.name))
               return(out)
             }
@@ -427,10 +440,14 @@ setMethod('predict2', signature(object = 'EMcv_biomod2_model', newdata = "SpatRa
           }
 )
 
+##' 
 ##' @rdname predict2.em
+##' 
+
 setMethod('predict2', signature(object = 'EMcv_biomod2_model', newdata = "data.frame"),
-          function(object, newdata, ...) {
-            predfun <- function(newdata, na.rm, ...){
+          function(object, newdata, ...)
+          {
+            predfun <- function(newdata, na.rm, ...) {
               if (ncol(newdata) <= 1) {
                 stop(paste0("\n Model EMcv was not computed because only one single model was kept in ensemble modeling ("
                             , colnames(newdata), ")"))
@@ -438,7 +455,7 @@ setMethod('predict2', signature(object = 'EMcv_biomod2_model', newdata = "data.f
               out <- apply(newdata, 1,
                            function(x) {
                              ifelse(length(x) == 1, 0, 
-                                    sd(x, na.rm = na.rm)/mean(x, na.rm = na.rm)*100)
+                                    sd(x, na.rm = na.rm) / mean(x, na.rm = na.rm) * 100)
                            })
               return(out)
             } 
@@ -470,57 +487,60 @@ setClass('EMci_biomod2_model',
 ##' @rdname predict2.em
 ##' 
 
-
 setMethod('predict2', signature(object = 'EMci_biomod2_model', newdata = "SpatRaster"),
-          function(object, newdata, ...) {
-            predfun <- function(newdata, on_0_1000, mod.name, na.rm, ...){
+          function(object, newdata, ...)
+          {
+            predfun <- function(newdata, on_0_1000, mod.name, na.rm, ...) {
               args <- list(...)
               mean_prediction <- args$mean_prediction
               sd_prediction <- args$sd_prediction
               side <- args$side
-
+              data.type <- args$data.type
+              
               if (is.null(mean_prediction)) { 
-                mean_prediction <- app(newdata, mean, wopt = list(names = mod.name),
-                                       na.rm = na.rm)
+                mean_prediction <- app(newdata, mean, wopt = list(names = mod.name), na.rm = na.rm)
               }
               if (is.null(sd_prediction)) { 
-                sd_prediction <- app(newdata, sd, wopt = list(names = mod.name),
-                                     na.rm = na.rm) 
+                sd_prediction <- app(newdata, sd, wopt = list(names = mod.name), na.rm = na.rm) 
               }
               
               ci_prediction <-  switch(
                 side,
-                inferior = mean_prediction -  (sd_prediction * (qt((1-object@alpha/2), df = length(object@model) + 1 ) / sqrt(length(object@model))) ),
-                superior = mean_prediction +  (sd_prediction * (qt((1-object@alpha/2), df = length(object@model) + 1 ) / sqrt(length(object@model))) )
+                inferior = mean_prediction - (sd_prediction * (qt((1 - object@alpha / 2), df = length(object@model) + 1) / sqrt(length(object@model)))),
+                superior = mean_prediction + (sd_prediction * (qt((1 - object@alpha / 2), df = length(object@model) + 1) / sqrt(length(object@model))))
               )
               
-              if (on_0_1000) {
-                ci_prediction <- classify(round(ci_prediction),
-                                          matrix(c(-Inf, 0, 0,
-                                                   1000, Inf, 1000),
-                                                 nrow = 2, byrow = TRUE))
-              } else {
-                ci_prediction <- classify(round(ci_prediction),
-                                          matrix(c(-Inf, 0, 0,
-                                                   1, Inf, 1),
-                                                 nrow = 2, byrow = TRUE))     
+              if (data.type == "binary") {
+                if (on_0_1000) {
+                  ci_prediction <- classify(round(ci_prediction),
+                                            matrix(c(-Inf, 0, 0, 1000, Inf, 1000),
+                                                   nrow = 2, byrow = TRUE))
+                } else {
+                  ci_prediction <- classify(round(ci_prediction),
+                                            matrix(c(-Inf, 0, 0, 1, Inf, 1),
+                                                   nrow = 2, byrow = TRUE))
+                }
               }
               ci_prediction
             } 
-            
             # redirect to predict2.biomod2_ensemble_model.SpatRaster
-            callNextMethod(object, newdata, predfun = predfun, side = object@side, ...)
+            callNextMethod(object, newdata, predfun = predfun, side = object@side, data.type = object@model_type, ...)
           }
 )
 
+##' 
 ##' @rdname predict2.em
+##' 
+
 setMethod('predict2', signature(object = 'EMci_biomod2_model', newdata = "data.frame"),
-          function(object, newdata, ...) {
-            predfun <- function(newdata, na.rm, ...){
+          function(object, newdata, ...)
+          {
+            predfun <- function(newdata, na.rm, ...) {
               args <- list(...)
               mean_prediction <- args$mean_prediction
               sd_prediction <- args$sd_prediction
               side <- args$side
+              data.type <- args$data.type
               on_0_1000 <- args$on_0_1000
               
               if (is.null(mean_prediction)) { 
@@ -531,22 +551,24 @@ setMethod('predict2', signature(object = 'EMci_biomod2_model', newdata = "data.f
               }
               
               ci_prediction <- switch(side,
-                                      inferior = mean_prediction -  (sd_prediction * (qt((1-object@alpha/2), df = length(object@model) + 1 ) / sqrt(length(object@model))) ),
-                                      superior = mean_prediction +  (sd_prediction * (qt((1-object@alpha/2), df = length(object@model) + 1 ) / sqrt(length(object@model))) ))
+                                      inferior = mean_prediction - (sd_prediction * (qt((1 - object@alpha / 2), df = length(object@model) + 1) / sqrt(length(object@model)))),
+                                      superior = mean_prediction + (sd_prediction * (qt((1 - object@alpha / 2), df = length(object@model) + 1) / sqrt(length(object@model)))))
               
               # reclassify prediction to prevent from out of bounds prediction
-              if (on_0_1000) {
-                ci_prediction <- round(ci_prediction * 1000)
-                ci_prediction[ci_prediction > 1000] <- 1000
-                ci_prediction[ci_prediction < 0] <- 0
-              } else {
-                ci_prediction[ci_prediction > 1] <- 1
-                ci_prediction[ci_prediction < 0] <- 0
+              if (data.type == "binary") {
+                if (on_0_1000) {
+                  ci_prediction <- round(ci_prediction * 1000)
+                  ci_prediction[ci_prediction > 1000] <- 1000
+                  ci_prediction[ci_prediction < 0] <- 0
+                } else {
+                  ci_prediction[ci_prediction > 1] <- 1
+                  ci_prediction[ci_prediction < 0] <- 0
+                }
               }
               ci_prediction
             }
             # redirect to predict2.biomod2_ensemble_model.data.frame
-            callNextMethod(object, newdata, predfun = predfun, side = object@side, ...)
+            callNextMethod(object, newdata, predfun = predfun, side = object@side, data.type = object@model_type, ...)
           }
 )
 
@@ -569,26 +591,22 @@ setClass('EMca_biomod2_model',
 ##' 
 
 setMethod('predict2', signature(object = 'EMca_biomod2_model', newdata = "SpatRaster"),
-          function(object, newdata, data_as_formal_predictions = FALSE, ...) {
+          function(object, newdata, data_as_formal_predictions = FALSE, ...)
+          {
             args <- list(...)
             on_0_1000 <- args$on_0_1000
-            if (is.null(on_0_1000)) { 
-              on_0_1000 <- FALSE
-            }
+            if (is.null(on_0_1000)) { on_0_1000 <- FALSE }
             
-            predfun <- function(newdata, on_0_1000, thresh, mod.name, na.rm, ...){
+            predfun <- function(newdata, on_0_1000, thresh, mod.name, na.rm, ...) {
               if (nlyr(newdata) == 1) {
                 return(bm_BinaryTransformation(newdata, thresh))
               } else {
                 return(
-                  app(bm_BinaryTransformation(newdata, thresh),
-                      function(x){
-                        m <- mean(x, na.rm = na.rm)
-                        if (on_0_1000) { 
-                          m <- round(m * 1000)
-                        }
-                        return(m)
-                      }, wopt = list(names = mod.name))
+                  app(bm_BinaryTransformation(newdata, thresh), function(x) {
+                    m <- mean(x, na.rm = na.rm)
+                    if (on_0_1000) { m <- round(m * 1000) }
+                    return(m)
+                  }, wopt = list(names = mod.name))
                 )
               }
             }
@@ -597,29 +615,26 @@ setMethod('predict2', signature(object = 'EMca_biomod2_model', newdata = "SpatRa
             } else {
               thresh <- object@thresholds / 1000 
             }
-            
-
             # redirect to predict2.biomod2_ensemble_model.SpatRaster
             callNextMethod(object, newdata, predfun = predfun, thresh = thresh,
-                           data_as_formal_predictions = data_as_formal_predictions,
-                           ...)
+                           data_as_formal_predictions = data_as_formal_predictions, ...)
           }
 )
 
+##' 
 ##' @rdname predict2.em
+##' 
+
 setMethod('predict2', signature(object = 'EMca_biomod2_model', newdata = "data.frame"),
-          function(object, newdata, data_as_formal_predictions = FALSE, ...) {
+          function(object, newdata, data_as_formal_predictions = FALSE, ...)
+          {
             args <- list(...)
             on_0_1000 <- args$on_0_1000
-            if (is.null(on_0_1000)) { 
-              on_0_1000 <- FALSE 
-            }
-            predfun <- function(newdata, na.rm, ...){
-              out <- rowMeans(bm_BinaryTransformation(newdata, thresh), 
-                              na.rm = na.rm)
-              if (on_0_1000) {
-                out <- round(out * 1000)
-              }
+            if (is.null(on_0_1000)) { on_0_1000 <- FALSE }
+            
+            predfun <- function(newdata, na.rm, ...) {
+              out <- rowMeans(bm_BinaryTransformation(newdata, thresh), na.rm = na.rm)
+              if (on_0_1000) { out <- round(out * 1000) }
               out
             }
             if (on_0_1000) { 
@@ -627,12 +642,9 @@ setMethod('predict2', signature(object = 'EMca_biomod2_model', newdata = "data.f
             } else {
               thresh <- object@thresholds / 1000
             }
-            
-
             # redirect to predict2.biomod2_ensemble_model.data.frame
             callNextMethod(object, newdata, predfun = predfun,
-                           data_as_formal_predictions = data_as_formal_predictions,
-                           ...)
+                           data_as_formal_predictions = data_as_formal_predictions, ...)
           }
 )
 
@@ -645,7 +657,7 @@ setMethod('predict2', signature(object = 'EMca_biomod2_model', newdata = "data.f
 ##' @export
 
 setClass('EMwmean_biomod2_model',
-         representation(penalization_scores='numeric'),
+         representation(penalization_scores = 'numeric'),
          contains = 'biomod2_ensemble_model',
          prototype = list(model_class = 'EMwmean'),
          validity = function(object) { return(TRUE) })
@@ -655,60 +667,166 @@ setClass('EMwmean_biomod2_model',
 ##' 
 
 setMethod('predict2', signature(object = 'EMwmean_biomod2_model', newdata = "SpatRaster"),
-          function(object, newdata, data_as_formal_predictions = FALSE, ...) {
+          function(object, newdata, data_as_formal_predictions = FALSE, ...)
+          {
             if (ncol(newdata) < 1) {
               stop("Model EMwmean was not computed because no single model was kept in ensemble modeling")
             }
-            predfun <- function(newdata, on_0_1000, penalization_scores,
-                                mod.name, na.rm, ...){
+            predfun <- function(newdata, on_0_1000, penalization_scores, mod.name, ...) {
               if (nlyr(newdata) == 1) {
                 return(newdata)
               } else {
                 return(
                   app(newdata, function(x) {
-                    wm <- sum(x * penalization_scores, na.rm = na.rm)
-                    if (on_0_1000) { 
-                      wm <- round(wm) 
-                    }
+                    wm <- sum(x * penalization_scores)
+                    if (on_0_1000) { wm <- round(wm) }
                     return(wm)
                   }, wopt = list(names = mod.name))
                 )
               }
             }
-
             # redirect to predict2.biomod2_ensemble_model.SpatRaster
-            callNextMethod(object, newdata,
-                           predfun = predfun,
+            callNextMethod(object, newdata, predfun = predfun,
                            data_as_formal_predictions = data_as_formal_predictions,
                            penalization_scores = object@penalization_scores, ...)
           }
 )
 
+##' 
 ##' @rdname predict2.em
+##' 
+
 setMethod('predict2', signature(object = 'EMwmean_biomod2_model', newdata = "data.frame"),
-          function(object, newdata, data_as_formal_predictions = FALSE, ...) {
-            
+          function(object, newdata, data_as_formal_predictions = FALSE, ...)
+          {
             if (ncol(newdata) < 1) {
               stop("Model EMwmean was not computed because no single model was kept in ensemble modeling")
             }
-            predfun <- function(newdata, on_0_1000, penalization_scores, na.rm, ...){
-              # out <- as.vector(
-              #   as.matrix(newdata) %*% penalization_scores
-              # )
-              out <- as.vector(rowSums(t(apply(as.matrix(newdata), 1, function(x) penalization_scores*x)), na.rm = na.rm))
-              if (on_0_1000) { 
-                out <- round(out) 
+            predfun <- function(newdata, on_0_1000, penalization_scores, na.rm, ...) {
+              out <- apply(as.matrix(newdata), 1, function(x) penalization_scores*x)
+              if (ncol(newdata) == 1) {
+                out <- matrix(out, nrow = 1)
               }
+              out <- as.vector(rowSums(t(out), na.rm = na.rm))
+              if (on_0_1000) { out <- round(out) }
               out
             }
-            
-
             # redirect to predict2.biomod2_ensemble_model.SpatRaster
-            callNextMethod(object, newdata,
-                           predfun = predfun, 
+            callNextMethod(object, newdata, predfun = predfun, 
                            penalization_scores = object@penalization_scores,
                            data_as_formal_predictions = data_as_formal_predictions,
                            ...)
           }
 )
 
+### -------------------------------------------------------------------------- #
+### 10.7 EMmode_biomod2_model ------------------------------------------------
+### -------------------------------------------------------------------------- #
+##' @name EMmode_biomod2_model-class
+##' @rdname biomod2_ensemble_model
+##' @export
+
+setClass('EMmode_biomod2_model',
+         representation(),
+         contains = 'biomod2_ensemble_model',
+         prototype = list(model_class = 'EMmode'),
+         validity = function(object) { return(TRUE) })
+
+##' 
+##' @rdname predict2.em
+##' 
+
+setMethod('predict2', signature(object = 'EMmode_biomod2_model', newdata = "SpatRaster"),
+          function(object, newdata, ...)
+          {
+            predfun <- function(newdata, on_0_1000, mod.name, na.rm, ...) {
+              if (nlyr(newdata) == 1) {
+                return(newdata)
+              } else {
+                # return(
+                #   app(newdata, .find_freq_mode, wopt = list(names = mod.name))
+                # )
+              
+                levels <- unlist(unique(values(newdata[[1]], dataframe = T, na.rm = T)))## On assume qu'il les a tous....
+                levels <- levels(levels)
+                
+                pred <- app(newdata, function(x){
+                  m <- .find_mode(x)
+                  m <- as.numeric(m)
+                  return(m)
+                }, wopt = list(names = mod.name))
+                pred <- terra::subst(pred, 1:length(levels), levels)
+                return(pred)
+              }
+            }
+            # redirect to predict2.biomod2_ensemble_model.SpatRaster
+            callNextMethod(object, newdata, predfun = predfun, ...)
+          }
+)
+
+##' 
+##' @rdname predict2.em
+##' 
+
+setMethod('predict2', signature(object = 'EMmode_biomod2_model', newdata = "data.frame"),
+          function(object, newdata, ...)
+          {
+            predfun <- function(newdata, on_0_1000, na.rm, ...) {
+              out <- unlist(apply(newdata, 1, .find_mode))
+              out
+            }
+            # redirect to predict2.biomod2_ensemble_model.SpatRaster
+            callNextMethod(object, newdata, predfun = predfun, ...)
+          }
+)
+
+
+### -------------------------------------------------------------------------- #
+### 10.8 EMfreq_biomod2_model ------------------------------------------------
+### -------------------------------------------------------------------------- #
+##' @name EMfreq_biomod2_model-class
+##' @rdname biomod2_ensemble_model
+##' @export
+
+setClass('EMfreq_biomod2_model',
+         representation(),
+         contains = 'biomod2_ensemble_model',
+         prototype = list(model_class = 'EMfreq'),
+         validity = function(object) { return(TRUE) })
+
+##' 
+##' @rdname predict2.em
+##' 
+
+setMethod('predict2', signature(object = 'EMfreq_biomod2_model', newdata = "SpatRaster"),
+          function(object, newdata, ...)
+          {
+            predfun <- function(newdata, on_0_1000, mod.name, na.rm, ...) {
+              if (nlyr(newdata) == 1) {
+                return(newdata)
+              } else {
+                return(
+                  app(newdata, .find_freq_mode, wopt = list(names = mod.name))
+                )
+              }
+            }
+            # redirect to predict2.biomod2_ensemble_model.SpatRaster
+            callNextMethod(object, newdata, predfun = predfun, ...)
+          }
+)
+
+##' 
+##' @rdname predict2.em
+##' 
+
+setMethod('predict2', signature(object = 'EMfreq_biomod2_model', newdata = "data.frame"),
+          function(object, newdata, ...)
+          {
+            predfun <- function(newdata, on_0_1000, na.rm, ...) {
+              out <- unlist(apply(newdata, 1, .find_freq_mode))
+              out
+            }
+            # redirect to predict2.biomod2_ensemble_model.SpatRaster
+            callNextMethod(object, newdata, predfun = predfun, ...)
+          }
+)
\ No newline at end of file
